package ACL2::Orden::Includebook;
use base ACL2::Orden;

#
#    Orden INCLUDE-BOOK
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Tags;
use ACL2::Localizacion;
use Carp;

my $lh = ACL2::Localizacion->get_handle;

=head2 get_nombre_libro($this)

Uso: ME<eacute>todo pE<uacute>blico.
Responsabilidades: Devuelve el nombre corto (ruta no completa) del
libro incluido.
Entrada: Ninguna.
Salida: Cadena con el nombre corto del libro incluido en la orden.

=cut

sub get_nombre_libro
{
    return $_[0]->{nombre_libro};
}

=head2 get_book_path

Returns the path to this book.

=cut

sub get_book_path
{
    my ($self) = @_;

    # If :DIR is not specified, use the CBD. Otherwise, consult the
    # ACL2 global state using the directory keyword which was passed.
    my $basepath = (exists $self->{params}->{":DIR"})
        ? $self->get_state->get_book_dir_by_name($self->{params}->{":DIR"})
        : $self->get_state->get_cbd;

    my $fullpath = File::Spec->catfile(
        $basepath, $self->get_nombre_libro . ".lisp");
    $fullpath = File::Spec->catfile(
        $basepath, lc $self->get_nombre_libro . ".lisp")
        if ! -f $fullpath;

    croak $lh->maketext(
        "No se pudo localizar el fichero de certificación del libro [_1]",
        $self->get_nombre_libro)
        if ! -f $fullpath;

    return $fullpath;
}

sub protegido_inicializar_sexpr
{
    my ($self) = @_;

    my $rv_formula  = $self->get_lisp->value;
    my $len_formula = scalar @$rv_formula;

    # Remove quotes
    my $nombreLibro       =  $rv_formula->[1]->value;
    $nombreLibro          =~ s/\"//g;
    $_[0]->{nombre_libro} =  $nombreLibro;

    # String->string hash for the rest of the parameters
    $self->{params} = {
        map { $_->value } @{$rv_formula}[2..$len_formula-1]
    };
}

sub protegido_inicializar_salida
{
    my ($this) = @_;

    # Intentamos obtener la ruta al LISP
    #
    # Esto no siempre funciona: no se muestra dicha cadena cuando se
    # hace la inclusión desde un CERTIFY-BOOK. En dicho caso, nos
    # limitamos a añadir .lisp al nombre del libro.
    my ($ruta_objeto) = ${$this->get_salida} =~ /Loading (.*?)\.o/;

    my ($vol, $dir, $fich) = File::Spec->splitpath($ruta_objeto
                                                || $this->get_nombre_libro);

    # Probamos con minúsculas y mayúsculas
    my $ruta_libro  = File::Spec->catpath($vol, $dir, $fich . ".lisp");
    $ruta_libro  = File::Spec->catpath($vol, lc $dir, lc $fich . ".lisp")
        if ! -f $ruta_libro;

    # Buscamos el archivo de certificado .acl2 también
    my $ruta_cert = File::Spec->catpath($vol, $dir, $fich . ".acl2");
    $ruta_cert = File::Spec->catpath($vol, lc $dir, lc $fich . ".acl2")
        if ! -f $ruta_cert;
    $ruta_cert = File::Spec->catpath($vol, $dir, "cert.acl2")
        if ! -f $ruta_cert;

    # Comprobamos que ambas rutas existen
    croak $lh->maketext("No se pudo localizar el libro [_1]",
                        $ruta_libro)
        if ! -f $ruta_libro;
    croak $lh->maketext("No se pudo localizar el fichero de certificación del libro [_1]",
                        $ruta_cert)
        if ! -f $ruta_cert;

    $this->{ruta_libro} = $ruta_libro;
    $this->{ruta_cert}  = $ruta_cert;
}

sub protegido_cabecera
{
    $_[1]->startTag(ACL2::Tags::INCLUDEBOOK_TAG,
                    "name" => $_[0]->{nombre_libro},
                    "path_book" => File::Spec->abs2rel($_[0]->{ruta_libro}),
                    "path_cert" => File::Spec->abs2rel($_[0]->{ruta_cert}));
}

sub protegido_cuerpo
{
    my ($this, $writer) = @_;
    ACL2::Orden->salida_parrafos($writer, ${$this->get_salida});
}

1;
